const orcl = require('../service/OracleService');

// Usage sample
// const constructQuery = require('./app/functions/ConstructQuery')
// async function teste(){
//     let table = await constructQuery('turcaixa',['CDFILIAL','CDCAIXA','DTABERCAIX'])
//     return table
// }
// teste().then( res => {
//     console.log( res )
// })

async function constructQuery(tablename, wherefields = []) {
  const query = `select * from ${tablename} where 2 < 1`;

  orcl.oracledb.extendedMetaData = true;
  const orFields = await orcl.fetch(query, []);
  orcl.oracledb.extendedMetaData = false;

  const queryPrimaryKey = `SELECT COLUMN_NAME FROM ALL_CONS_COLUMNS WHERE CONSTRAINT_NAME = 'PK_${tablename}'  GROUP BY COLUMN_NAME, POSITION ORDER BY POSITION`;

  orcl.oracledb.extendedMetaData = true;
  const primaryKey = await orcl.fetch(queryPrimaryKey, []);
  orcl.oracledb.extendedMetaData = false;

  const columnPrimaryKey = [];

  for (const columnName of primaryKey.rows) {
    columnPrimaryKey.push(columnName[0]);
  }

  let into = '';
  let values = '';
  let fields = [];
  let fieldAndValues = '';

  let updateWhere = '';

  if (typeof wherefields === 'object') {
    let argCount = 0;

    for (const item of wherefields) {
      if (item.search('DT') === 0 || item.search('DT') === 2) {
        updateWhere += !argCount
          ? `${item} = TIMESTAMP :${item} `
          : `and ${item} = TIMESTAMP :${item} `;
      } else {
        updateWhere += !argCount ? `${item} = :${item} ` : `and ${item} = :${item} `;
      }

      argCount += 1;
    }
  } else {
    updateWhere = wherefields;
  }

  if (orFields.metaData.length) {
    for (const field of orFields.metaData) {
      into += into === '' ? '' : ', ';
      into += field.name;

      values += values === '' ? '' : ', ';
      fieldAndValues += fieldAndValues === '' ? '' : ' , ';

      const campoChave = columnPrimaryKey.find((coluna) => coluna === field.name);
      if (field.dbTypeName === 'DATE') {
        values += `TIMESTAMP :${field.name}`;
        if (campoChave !== field.name) {
          fieldAndValues += `${field.name} = TIMESTAMP :${field.name}`;
        } else {
          fieldAndValues = fieldAndValues.slice(0, -3);
        }
      } else {
        values += `:${field.name}`;
        if (campoChave !== field.name) {
          fieldAndValues += `${field.name} = :${field.name}`;
        } else {
          fieldAndValues = fieldAndValues.slice(0, -3);
        }
      }
    }
  }

  fields = orFields.metaData;

  const insertQuery = `INSERT INTO ${tablename} (${into}) VALUES(${values})`;
  let updateQuery = `UPDATE ${tablename} SET ${fieldAndValues} `;
  updateQuery += updateWhere ? ` WHERE ${updateWhere}` : '';

  return { insertQuery, updateQuery, fields };
}

module.exports = constructQuery;
