const updaterProdutosModel = require('../../model/updaterProdutosModel');
const updaterProdVersaoModel = require('../../model/updaterProdVersaoModel');
const updaterHistoricoModel = require('../../model/updaterHistoricoModel');
const socketService = require('../../service/SocketService');
const serverConfig = require('../../../../config/ServerConfig');

const route = (router) => {
  router.get('/updaterProdutos', (async (req, res) => {
    updaterProdutosModel.getUpdaterProdutos(true).then((products) => {
      const response = { 
        data: products.rows,
        pagination: {
          limit: 999,
          total: products.rows.length
        }
      };
      res.send(response);
    }).catch((error) => {
      res.send(error.message || error);
    });
  }));

  router.get('/updaterProdutosFilial', (async (req, res) => {
    const productId = req.query.PRODUCT_ID;
    if(!productId) {
      res.send([]);
    } else {
      updaterProdutosModel.getUpdaterProdutosWithUnits(productId).then(async (products) => {
        let clients = await socketService.getUpdaterSocket().getClients();
        const connectedFiliais = clients.map((item) => {
          return item.socket.CDFILIAL;
        });
        products.rows = products.rows.map((product) => {
          product.CONNECTION_STATUS = connectedFiliais.includes(product.CDFILIAL);
          return product;
        });
        const response = { 
          data: products.rows,
          pagination: {
            limit: 999,
            total: products.rows.length          
          }
        };
        res.send(response);
      }).catch((error) => {
        res.send(error.message || error);
      });
    }
  }));

  router.get('/updaterProdutosVersao', (async (req, res) => {
    const productId = req.query.PRODUCT_ID;
    if(!productId) {
      res.send([]);
    } else {
      updaterProdVersaoModel.getUpdaterProdVersaoByProduct(productId, serverConfig.useVersionFilter).then((products) => {
        const response = { 
          data: products.rows,
          pagination: {
            limit: 999,
            total: products.rows.length
          }
        };
        res.send(response);
      }).catch((error) => {
        res.send(error.message || error);
      });
    }
  }));

  router.get('/updaterProdutosVersaoConfig', (async (req, res) => {
    updaterProdVersaoModel.getUpdaterProdVersaoConfig().then((products) => {
      const response = { 
        data: products.rows,
        pagination: {
          limit: 999,
          total: products.rows.length
        }
      };
      res.send(response);
    }).catch((error) => {
      res.send(error.message || error);
    });
  }));

  router.get('/updaterProdutoHistorico', (async (req, res) => {
    const cdProductId = req.query.CDPRODUCTID;
    const cdFilial = req.query.CDFILIAL;
    updaterHistoricoModel.getUpdaterProdutoHistorico(cdProductId, cdFilial).then((historico) => {
      const response = { 
        data: historico.rows,
        pagination: {
          limit: 999,
          total: historico.rows.length
        }
      };
      res.send(response);
    }).catch((error) => {
      res.send(error.message || error);
    });
  }));

  router.put('/updateUpdaterProdVersaoAprov', (async (req, res) => {
    try {
      const result = await updaterProdVersaoModel.updateUpdaterProdVersaoAprov({
        ID: req.body.ID,
        IDAPROVVER: req.body.IDAPROVVER,
      });
      
      result.connection.commit();
      res.send('Sucesso!');
    } catch (error) {
      res.send(error.message || error);
    }
  }));
}

module.exports = route;