const updaterModel = require("../model/updaterModel");
const updaterProdFiliModel = require("../model/updaterProdFiliModel");
const GeraNovoCodigo = require("../modules/NovoCodigoModule");
const OracleService = require("./OracleService");
const Base64 = require("../functions/Base64");
const { v4: uuidv4 } = require("uuid");

class UpdaterService {
  async insertEventTable(cdFilial, cdOperador, devopsId, prodFiliId, currentVersion, installVersion, createNewVersion, action) {
    let response = {
      error: true,
      message: "O evento já foi registrado no sistema.",
    };
   
    let params = {
      P_CDFILIAL: cdFilial,
      P_CDPRODUCTID: devopsId,
    };

    const unitEvents = await updaterModel.getEventsByUnit(params);

    if (unitEvents.rows.length === 0) {
      const eventSeq = await GeraNovoCodigo.prxUpdaterEvento(cdFilial);
      const uuid = uuidv4();

      params = {
        P_CDFILIAL: cdFilial,
        P_CDEVENTO: eventSeq,
        P_CDUUID: uuid,
        P_CDPRODUCTID: devopsId,
        P_CDPRODFILIID: prodFiliId,
        P_IDSTEVENTO: action,
        P_NRVERSAOATUAL: currentVersion,
        P_NRVERSAOALVO: installVersion,
        P_IDCRIARVERSAO: createNewVersion,
      };

      await updaterModel.insertPendingEvent(params);

      const historySeq = await GeraNovoCodigo.prxUpdaterHistorico(cdFilial);

      params = {
        P_CDFILIAL: cdFilial,
        P_CDHISTORICO: historySeq,
        P_CDEVENTO: eventSeq,
        P_CDUUID: uuid,
        P_CDOPERADOR: cdOperador,
        P_CDPRODUCTID: devopsId,
        P_NMPRODUCT: null,
        P_IDSTEVENTO: action,
        P_NRVERSAOATUAL: currentVersion,
        P_NRVERSAOALVO: installVersion,
        P_IDCRIARVERSAO: createNewVersion,
      };

      await updaterModel.appendUpdaterHistory(params);

      response = {
        error: false,
        message: "Evento cadastrado com sucesso.",
      };

      await OracleService.commit();
    }

    return response;
  }

  async updateEventTable(cdFilial, cdOperador, devopsId, currentVersion, installVersion, createNewVersion, action) {
    let response = {
      error: true,
      message: "O evento já foi atualizado no sistema.",
    };

    const eventId = action === "INSTALACAO_EM_CURSO" ? "INSTALACAO_PENDENTE" : "REMOCAO_PENDENTE";

    let params = {
      P_CDFILIAL: cdFilial,
      P_CDPRODUCTID: devopsId,
      P_IDSTEVENTO: eventId,
    };

    const pendingEvents = await updaterModel.getProductUpdaterEventByEventId(params);

    if (pendingEvents.rows.length > 0) {
      const uuid = uuidv4();

      params = {
        P_IDSTEVENTO: action,
        P_CDEVENTO: pendingEvents.rows[0].CDEVENTO,
      };

      await updaterModel.updateEventInProgress(params);

      const history = await updaterModel.getUpdaterHistoryByEventId(params);

      if (history.rows.length === 0) {
        const historySeq = await GeraNovoCodigo.prxUpdaterHistorico(cdFilial);

        params = {
          P_CDFILIAL: cdFilial,
          P_CDHISTORICO: historySeq,
          P_CDEVENTO: pendingEvents.rows[0].CDEVENTO,
          P_CDUUID: uuid,
          P_CDOPERADOR: cdOperador,
          P_CDPRODUCTID: devopsId,
          P_NMPRODUCT: null,
          P_IDSTEVENTO: action,
          P_NRVERSAOATUAL: currentVersion,
          P_NRVERSAOALVO: installVersion,
          P_IDCRIARVERSAO: createNewVersion,
        };

        await updaterModel.appendUpdaterHistory(params);

        response = {
          error: false,
          message: "Evento atualizado com sucesso. Histórico criado",
        };
      }
      await OracleService.commit();
    }

    return response;
  }

  async finishEventTable(cdFilial, cdOperador, devopsId, prodFiliId, currentVersion, installVersion, createNewVersion) {
    let response = {
      error: true,
      message: "O evento já foi atualizado no sistema.",
    };

    let params = {
      P_CDFILIAL: cdFilial,
      P_CDPRODUCTID: devopsId,
    };

    const inProgressEvents = await updaterModel.getInProgressEvents(params);

    if (inProgressEvents.rows.length > 0) {
      const uuid = uuidv4();

      params = {
        P_IDSTEVENTO: "FINALIZADO",
        P_CDEVENTO: inProgressEvents.rows[0].CDEVENTO,
      };

      await updaterModel.updateEventInProgress(params);

      const history = await updaterModel.getUpdaterHistoryByEventId(params);

      if (history.rows.length === 0) {
        const historySeq = await GeraNovoCodigo.prxUpdaterHistorico(cdFilial);

        params = {
          P_CDFILIAL: cdFilial,
          P_CDHISTORICO: historySeq,
          P_CDEVENTO: inProgressEvents.rows[0].CDEVENTO,
          P_CDUUID: uuid,
          P_CDOPERADOR: cdOperador,
          P_CDPRODUCTID: devopsId,
          P_NMPRODUCT: null,
          P_IDSTEVENTO: "FINALIZADO",
          P_NRVERSAOATUAL: currentVersion,
          P_NRVERSAOALVO: installVersion,
          P_IDCRIARVERSAO: createNewVersion,
        };

        await updaterModel.appendUpdaterHistory(params);

        await this.updateProdFiliVersion(prodFiliId, installVersion);

        response = {
          error: false,
          message: "Evento atualizado com sucesso. Histórico criado",
        };
      }

      await OracleService.commit();
    }

    return response;
  }

  async updateProdFiliVersion(prodFiliId, updateVersion) {
    const params = {
      NRVERSAO: updateVersion,
      ID: prodFiliId,
    };

    await updaterProdFiliModel.updateUpdaterProdFiliVersion(params);
  }

  async cancelUnitEvents(cdFilial) {
    let params = {
      P_CDFILIAL: cdFilial,
    };

    await updaterModel.cancelUnitEvents(params);

    await OracleService.commit();
  }

  async cancelEventTable(cdFilial, cdOperador, devopsId, currentVersion, installVersion, createNewVersion) {
    let response = {
      error: true,
      message: "O evento já foi atualizado no sistema.",
    };

    let params = {
      P_CDFILIAL: cdFilial,
      P_CDPRODUCTID: devopsId,
    };

    const unitEvents = await updaterModel.getEventsByUnit(params);

    if (unitEvents.rows.length > 0) {
      const uuid = uuidv4();

      params = {
        P_IDSTEVENTO: "CANCELADO",
        P_CDEVENTO: unitEvents.rows[0].CDEVENTO,
      };

      await updaterModel.updateEventInProgress(params);

      const history = await updaterModel.getUpdaterHistoryByEventId(params);

      if (history.rows.length === 0) {
        const historySeq = await GeraNovoCodigo.prxUpdaterHistorico(cdFilial);

        params = {
          P_CDFILIAL: cdFilial,
          P_CDHISTORICO: historySeq,
          P_CDEVENTO: unitEvents.rows[0].CDEVENTO,
          P_CDUUID: uuid,
          P_CDOPERADOR: cdOperador,
          P_CDPRODUCTID: devopsId,
          P_NMPRODUCT: null,
          P_IDSTEVENTO: "CANCELADO",
          P_NRVERSAOATUAL: currentVersion,
          P_NRVERSAOALVO: installVersion,
          P_IDCRIARVERSAO: createNewVersion,
        };

        await updaterModel.appendUpdaterHistory(params);

        response = {
          error: false,
          message: "Evento atualizado com sucesso. Histórico criado",
        };
      }
      await OracleService.commit();
    }

    return response;
  }

  async getPendingEvents() {
    return updaterModel.getPendingEvents();
  }

  async getPendingEventsByUnit(cdFilial) {
    try {
      const params = {
        P_CDFILIAL: cdFilial,
      };
  
      const pendingEvents = await updaterModel.getPendingEventsByUnit(params);
  
      if (pendingEvents.rows.length === 0) {
        throw 'Nenhum evento pendente de instalação/remoção encontrado.';
      }
      
      return {
        error: false,
        message: 'Nenhum evento pendente de instalação/remoção encontrado.',
        events: pendingEvents.rows
      }
    } catch (error) {
      return {
        error: true,
        message: error,
        events: {}
      }
    }
  }
  
  async checkVersion(params) {
    const result = await updaterModel.getCheckInstalledVersion(params);
    return { error: result.rows.length > 0 };
  }

  async saveConfig({ config }){
    try {
      if (!config) {
        throw {
          error: true,
          message: 'Codigo da Filial não foi informado'
        };
      }

      const params = {
        P_CDFILIAL: config.environment.CDFILIAL
      };
      
      let configIni = await updaterModel.getConfigIni(params);

      if (configIni.rows.length > 0) {
        throw {
          error: false,
          message: `Arquivo config.ini da Filial ${params.P_CDFILIAL} já está salvo na base de dados.`
        };
      };

      const configIniBase64 = Base64.encode(config);         
      
      if (!configIniBase64) {
        throw {
          error: true,
          message: 'Não foi possivel converter config.ini da filial para base64.'
        };
      }
      
      params.P_CONFIG = configIniBase64;

      const insertConfig = await updaterModel.insertConfigIni(params);

      if (insertConfig.rowsAffected === 0) {
        throw {
          error: true,
          message: 'Erro ao inserir as informações do config.ini na base de dados.'
        };
      }

      configIni = await updaterModel.getConfigIni(params);

      if (!configIni) {
        throw {
          error: true,
          message: 'Não foi possivel recuperar as informações do config.ini da filial.'
        };
      }

      const operator = await updaterModel.getDefaultOperator(params);

      if (operator.rows.length === 0 || operator.rows[0].CDOPERADOR === null) {
        throw {
          error: true,
          message: `Não foi possivel buscar o operador padrão da filial ${params.P_CDFILIAL}.`
        };
      }

      const cdOperador = operator.rows[0].CDOPERADOR;
      const updaterConfigId = configIni.rows[0].ID;

      if(!cdOperador && !updaterConfigId) {
        throw {
          error: true,
          message: 'Não foi possivel inserir os dados na tabela Historico'
        };
      }

      params.P_CDOPERADOR = cdOperador;
      params.P_UPDATERCONFIGID = updaterConfigId;

      await updaterModel.appendConfigIniHistory(params);         

      await OracleService.commit();       
      
      return {
        error: false,
        message: `Config.ini da filial ${params.P_CDFILIAL} foi inserido com sucesso`
      }
    } catch(error) {     
      await OracleService.rollback();      
      
      return error;
    }     
  }

  async getUnitConfigIniData(params) {
    try {
      const result = await updaterModel.getConfigIniByUnit(params);
  
      if (result.rows.length === 0) {
        throw new Error(`Não existe um arquivo de configuração salvo para a filial ${params.P_CDFILIAL}`);
      }

      return result.rows[0];
    } catch (error) {
      throw error;
    }
  }

  async getConfigIniByUnit(cdFilial) {
    try {
      const params = {
        P_CDFILIAL: cdFilial,
      };

      const result = await this.getUnitConfigIniData(params);
  
      const decodeConfig = Base64.decode(result.CONFIG);
      
      return {
        error: false,
        message: 'Arquivo de configuração carregado com sucesso!',
        data: decodeConfig
      }
    } catch (error) {
      return {
        error: true,
        message: error.message
      }
    }
  }

  async saveConfigIniByUnit(cdFilial, cdOperador, config) {
    try {
      const encodeConfig = Base64.encode(config);
      let result;

      let params = {
        P_CDFILIAL: cdFilial,
        P_CDOPERADOR: cdOperador,
        P_CONFIG: encodeConfig
      }
      
      result = await updaterModel.updateConfigIniByUnit(params);

      if (result.rowsAffected === 0) {
        throw new Error(`Não existe registro de um arquivo de configuração para a filial ${cdFilial}.`);
      }

      result = await this.getUnitConfigIniData(params);

      params.P_UPDATERCONFIGID = result.ID;

      result = await updaterModel.appendConfigIniHistory(params);
      
      if (result.rowsAffected === 0) {
        throw new Error(`Falha ao salvar o histórico para o arquivo de configuração da filial ${cdFilial}.`);
      }

      await OracleService.commit();
      
      return {
        error: false,
        message: 'Arquivo de configuração salvo com sucesso!'
      }
    } catch (error) {
      await OracleService.rollback();
      
      return {
        error: true,
        message: 'Erro ao salvar o arquivo de configuração. Error: ' + error.message
      }
    }
  }

  async getConfigHistoryByUnit(cdFilial) {
    try {
      let result;
      const params = {
        P_CDFILIAL: cdFilial,
      };

      result = await this.getUnitConfigIniData(params);
      
      params.P_ID = result.ID;
      
      result = await updaterModel.getConfigIniHistoryByUnit(params);
      
      return {
        error: false,
        message: 'Histórico de atualização do arquivo de configuração carregado com sucesso!',
        data: result.rows
      }
    } catch (error) {
      return {
        error: true,
        message: error.message
      }
    }
  }

  async getProdConfig(cdProductId, nrVersao) {
    try {
      const params = {
        CDPRODUCTID: cdProductId,
        NRVERSAO: nrVersao
      };

      if (!params.CDPRODUCTID) throw new Error('Codigo do produto não foi informado.'); 
        
      const { rows } = await updaterModel.getUpdaterProdConfig(params);

      if (!rows) throw new Error('Produto não foi encontrado.');

      const config = rows[0].CONFIG;

      if (config.includes("|")) {
        const resultConfig = config.split("|");
        const resultGetProdConfig = await updaterModel.getUpdaterProdConfigLastUpdate(params);    

      if (resultGetProdConfig.rows[0].CONFIG) {
        const decode = Base64.decode(resultGetProdConfig.rows[0].CONFIG);
        const resultConfigObject = this.resultNewProductConfiguration(resultConfig, decode);
        return {
          data: resultConfigObject,
          message: "Preencha os parâmetros do projeto antes de continuar instalação."
        };
      } else {
        return {
          data: resultConfig,
          message: "Preencha os parâmetros do projeto antes de continuar instalação."
        };
      }
    } 
        
    const toCheckBase64 = /^([A-Za-z0-9+/]{4})*([A-Za-z0-9+/]{4}|[A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{2}==)$/;
    if (toCheckBase64.test(config)) {
      const decodeProdConfig = Base64.decode(config);
      if (decodeProdConfig) {
        return {             
          data: decodeProdConfig
        }
      }
    }   
    } catch (error) {
      return {
        error: true,
        message: error.message
      }
    }
  }

  resultNewProductConfiguration(config, decode){
    let prodConfigObject = {};          
    
    config.map((element) => {      
      if (element in decode) {      
        prodConfigObject[element] = decode[element];
      } else {
        prodConfigObject[element] = '';
      }
    })  
    return prodConfigObject;   
  }

  async saveProdConfig(cdProductId, nrVersao, config){
    try {
      for (let item in config) {
        if (config[item] === '') {
          throw new Error('Ocorreu um erro ao salvar as informações, tente novamente.');
        }      
      }

      const encodeProdCondig = Base64.encode(config);

      if (!encodeProdCondig) {
        throw new Error(`Não foi possivel converter os parâmetros do produto ${params.CDPRODUCTID} para base64.`);
      }  
    
      const updaterProductConfig = {
        CDPRODUCTID: cdProductId, 
        NRVERSAO: nrVersao,     
        CONFIG: encodeProdCondig
      }

      const result = await updaterModel.setUpdateProdConfig(updaterProductConfig);
    
      if (result.rowsAffected === 0) {
        throw new Error(`Falha ao salvar parâmetros do produto ${params.CDPRODUCTID}.`);
      }    
    
      const decodeProdCondig = Base64.decode(encodeProdCondig);    

      await OracleService.commit();

      return {
        error: false,
        message: 'Parâmetros do projeto salvo com sucesso!',
        data: decodeProdCondig
      }
    } catch(error) {    
      await OracleService.rollback();
      
      return {
        error: true,
        message: error.message
      }
    }
  }

  async startServiceMonitor(services) {
    try {
      const serviceMonitorConfig = await this.getServiceMonitorConfiguration();
      const includeServicesArray = [];
      let monitorCommand = serviceMonitorConfig.COMANDO;

      for (let service of services) {
        if (monitorCommand.indexOf(service) === -1) {
          includeServicesArray.push(service);
        }
      }

      if (includeServicesArray.length > 0) {
        const regexRemoveSemicolonEndString = new RegExp(';$', 'g');
        monitorCommand = monitorCommand.replace(regexRemoveSemicolonEndString, '');

        const includeServices = includeServicesArray.join(';');
        monitorCommand = `${monitorCommand};${includeServices}`;

        const params = {
          P_ID: serviceMonitorConfig.ID,
          P_COMANDO: monitorCommand
        }
        
        await updaterModel.updateServiceMonitorConfigurationById(params);
        
        await OracleService.commit();
      }

      return {
        error: false,
        message: 'Monitoramento de serviços atualizado com sucesso!'
      }
    } catch (error) {
      return {
        error: true,
        message: error
      }
    }
  }

  async stopServiceMonitor(services) {
    try {
      const serviceMonitorConfig = await this.getServiceMonitorConfiguration();
      
      let monitorCommand = serviceMonitorConfig.COMANDO;

      for (const service of services) {
        const regexService = new RegExp(service, 'g');
        monitorCommand = monitorCommand.replace(regexService, '');
      }

      const regexRemoveDoubleSemicolon = new RegExp(';{2,}', 'g');
      monitorCommand = monitorCommand.replace(regexRemoveDoubleSemicolon, ';');
      
      const regexRemoveSemicolonEndString = new RegExp(';$', 'g');
      monitorCommand = monitorCommand.replace(regexRemoveSemicolonEndString, '');

      const params = {
        P_ID: serviceMonitorConfig.ID,
        P_COMANDO: monitorCommand
      }

      await updaterModel.updateServiceMonitorConfigurationById(params);
      
      await OracleService.commit();
      
      return {
        error: false,
        message: 'Monitoramento de serviços removidos com sucesso!'
      }
    } catch (error) {
      return {
        error: true,
        message: error
      }
    }
  }

  async getServiceMonitorConfiguration() {
    try {
      const result = await updaterModel.getServiceMonitorConfiguration();

      if (result.rows.length === 0) {
        throw "Não foi encontrado um monitoramento de serviços ativo.";
      }

      return result.rows[0];
    } catch (error) {
      throw error;
    }
  }
}

module.exports = new UpdaterService();
