const updaterModel = require("../model/updaterModel");
const base64 = require("../functions/Base64");
const semver = require("semver");
const supportedUpdaterVersion = "1.0.0";

class UpdaterSocket {
    clients = [];
    setSocketIo(io) {
        this.io = io;
    }
    getSocketIo() {
        return this.io;
    }
    validateVersionCompatibility(socket) {
        let updaterVersion = "1.0.0";
        if (semver.gt(supportedUpdaterVersion, updaterVersion)) {
            socket.emit("invalidVersion", {
                supportedUpdaterVersion: supportedUpdaterVersion,
                updaterVersion: updaterVersion,
            });

            console.log(
                `Disconnecting: (${info}): ${socket.id} - Versão do updater inferior a ${supportedUpdaterVersion}`
            );
            socket.disconnect();
            throw new Error("UPDATER_VERSION_INCOMPATIBLE");
        } else {
            socket.emit("validVersion");
        }
    }
    getRoom(roomName) {
        return this.getSocketIo().to(roomName);
    }
    async getClients() {
        // this.io.in('ASSISTANT_UPDATER_ROOM').fetchSockets().then(resp => console.log(resp))
        return this.clients;
    }
    addClient(client) {
        this.clients.push(client);
    }
    async removeClientBySocketId(socketId) {
        let clients = await this.getClients();
        let idxToRemove = clients.findIndex((cl) => cl.id === socketId);
        if (idxToRemove > -1) {
            clients.splice(idxToRemove, 1);
            return true;
        }
        return false;
    }

    async getSocketUpdaterByUnit(CDFILIAL) {
        return this.getSocketIo()
            .in("ASSISTANT_UPDATER_ROOM")
            .fetchSockets()
            .then((allSockets) => {
                const updaterUnitSocket = allSockets.find((socket) => {
                    return socket.CDFILIAL == CDFILIAL;
                });
                return updaterUnitSocket;
            });
    }

    registerEvents(socket) {
        socket.on("updateConfigResult", (resultConfigIni) => {
            this.getRoom("ASSISTANT_MASTER_ROOM").emit(
                "updateConfigIniFinish",
                resultConfigIni
            );
        });
        //Envia Filial para Updater
        socket.on("updateConfigInit", async (CDFILIAL) => {
            const socketUpdater = await this.getSocketUpdaterByUnit( CDFILIAL );
            if (socketUpdater) {
                try {
                    const { rows } = await updaterModel.getConfigIniByUnit({
                        P_CDFILIAL: CDFILIAL,
                    });

                    if (rows.length === 0) {
                        throw new Error(
                            `Não foi encontrado o registro do arquivo de configuração para a filial ${CDFILIAL}`
                        );
                    }

                    const configIni = rows[0].CONFIG;
                    const configIniDecode = base64.decode(configIni);

                    socketUpdater.emit("updateConfig", configIniDecode);
                } catch (error) {
                    this.getRoom("ASSISTANT_MASTER_ROOM").emit(
                        "updateConfigIniFinish",
                        {
                            error: true,
                            message: error.message,
                        }
                    );
                }
            } else {
                this.getRoom("ASSISTANT_MASTER_ROOM").emit(
                    "updateConfigIniFinish",
                    {
                        error: true,
                        message:
                            "Arquivo de configuração não pode ser atualizado na loja. Não existe conexão ativa com a loja no momento.",
                    }
                );
            }
        });

        socket.on("logUpdaterBranchResponse", (logs) => {
            this.getRoom("ASSISTANT_MASTER_ROOM").emit(
                "logUpdaterBranchFinish",
                logs
            );
        });

        socket.on("getLogUpdaterBranch", async (params) => {
            const updaterUnitSocket = await this.getSocketUpdaterByUnit(
                params.CDFILIAL
            );
            if (updaterUnitSocket)
                updaterUnitSocket.emit("getLogUpdaterBranch");
        });

        socket.on("updaterResultInProgress", (result) => {
            this.getRoom("ASSISTANT_MASTER_ROOM").emit(
                "updaterInProgress",
                result
            );
        });

        socket.on("updaterResultFinish", (result) => {
            this.getRoom("ASSISTANT_MASTER_ROOM").emit("updaterFinish", result);
        });

        socket.on("execUpdater", async (params) => {
            const updaterSocketUnit = await this.getSocketUpdaterByUnit(params.CDFILIAL);
            if (updaterSocketUnit) {
                // Envia requisição de atualização para a loja
                updaterSocketUnit.emit("updater", params);
            } else {
                console.log(
                    "CLIENT DO UPDATER NÃO ENCONTRADO NA FILIAL: " +
                        params.CDFILIAL
                );
            }
        });
    }
}

module.exports = new UpdaterSocket();